#!/usr/bin/env python
"""
unittest.py

Test that the translator is working
"""


__author__ = "Ardan Patwardhan, Sanja Abbott"
__email__ = "ardan@ebi.ac.uk, sanja@ebi.ac.uk"
__date__ = "2018-06-28"

import os.path
import unittest
import glob
from wwpdb.utils.emdb.cif_emdb_translator.cif_emdb_translator import CifEMDBTranslator


class TestTranslator(unittest.TestCase):
    """Run the translator on a bunch of example files"""

    testOutputXmlDir = "data/test/xml_v3_out"
    testInputCifDir = "data/cif"
    schema = "emdb.xsd"

    def test_cif2xml(self):

        # create the translator object
        translator = CifEMDBTranslator()
        translator.set_logger_logging(True, True, True, False)

        test_cifs = glob.glob(self.testInputCifDir + "/*.cif")
        for test_cif in test_cifs:
            print("In file: " + test_cif)
            out_filename = os.path.splitext(os.path.basename(test_cif))[0]
            xml_out = os.path.join(self.testOutputXmlDir + "/" + out_filename + ".xml")
            translator.translate_and_validate(test_cif, xml_out, self.schema)
            translator.write_logger_logs(True, True, True)


if __name__ == "__main__":
    unittest.main()



# Last update: 2019-05-17
import argparse
import sys

from mmcif_utils.trans.InstanceMapper import InstanceMapper
from wwpdb.utils.config.ConfigInfoApp import ConfigInfoAppEm
from wwpdb.utils.wf.plugins.UtilsBase import UtilsBase


class convert(UtilsBase):
    def __init__(self, inFile=None, outFile=None, verbose=False, log=sys.stderr):
        headerFilters = ["all", "prereleasetitle"]  # noqa: F841
        cI = ConfigInfoAppEm()
        self.mappingInfoPath = cI.get_emd_mapping_file_path()
        self.inFile = inFile
        self.outFile = outFile
        self.verbose = verbose
        self.log = log
        self.im = InstanceMapper(verbose=self.verbose, log=self.log)
        self.im.setMappingFilePath(self.mappingInfoPath)
        # self.im.setFilterTagList(headerFilters)

    def em2emd(self):
        ok = self.im.translate(self.inFile, self.outFile, mode="src-dst")  # noqa: F841 pylint: disable=unused-variable
        # print ok

    def emd2em(self):
        ok = self.im.translate(self.inFile, self.outFile, mode="dst-src")  # noqa: F841 pylint: disable=unused-variable
        # print ok


if __name__ == "__main__":
    parser = argparse.ArgumentParser(prog="em translate")
    parser.add_argument("-o", "--output", help="output file", type=str, required=True)
    parser.add_argument("-i", "--input", help="input file", type=str, required=True)
    parser.add_argument("-d", "--emToEmd", help="em to emd", action="store_true")
    parser.add_argument("-e", "--emdToEm", help="emd to em", action="store_true")

    args = parser.parse_args()

    if not sys.argv[1:]:
        parser.print_help()
        exit()

    if args.emToEmd:
        convert(inFile=args.input, outFile=args.output).em2emd()

    elif args.emdToEm:
        convert(inFile=args.input, outFile=args.output).emd2em()

    else:
        parser.print_help()
        exit()



##
# File:    EmdbSchema.py
# Date:    29-March-2021
"""
Wrapper for access to EMDB schema.

The schema is contained in this package
"""

import os


class EmdbSchema(object):
    def getSchemaPath(self):
        spath = os.path.join(os.path.dirname(__file__), "data", "emdb-v3.xsd")
        return os.path.abspath(spath)



__date__ = "2020-11-17"
__version__ = "0.1"


import os
from optparse import OptionParser

from wwpdb.utils.emdb.cif_emdb_translator.cif_emdb_translator import CifEMDBTranslator


def main():
    """
    This script calls CifEMDBTranslator to translate
    an mmcif emd file into an EMDB v3.x XML header file
    :return:
    """
    in_cif = None
    out_xml = None
    out_dir = "data/test/xml_v3_out/"
    schema = "emdb.xsd"
    usage = """
            test_cif_to_xml_translator.py [options]
            Convert an mmcif emd file to EMDB v3.x xml file

            Examples:
            python test_cif_to_xml_translator.py -i input_cif_file -o output_xml_file
            python test_cif_to_xml_translator.py -i "data/cif/EMD-0000.txt" -o "data/test/xml_v3_out/EMD-0000.xml"

            Typical run:
            python test_cif_to_xml_translator.py -i in.cif -o out.xml
                in.xml: a cif file in emd space
                out.cif: an EMDB XML file 3.x
            """
    parser = OptionParser(usage=usage, version=__version__)
    parser.add_option("-i", "--in-file", action="store", type="string", metavar="FILE", dest="in_cif", help="Input cif file")
    parser.add_option("-o", "--out-file", action="store", type="string", metavar="FILE", dest="out_xml", help="Output xml file")
    (options, args) = parser.parse_args()

    # if len(args) == 0:
    #     print(usage)
    #     sys.exit("No input options given!")

    if options.in_cif:
        in_cif = options.in_cif
    else:
        in_cif = "data/cif/EMD-0000.txt"
    if options.out_xml:
        out_xml = options.out_xml
    else:
        out_xml = out_dir + "EMD-0000.xml"

    # create the translator object
    translator = CifEMDBTranslator()
    # set up the translator's logger object
    translator.set_logger_logging(log_error=True, error_log_file_name=os.path.join(os.path.dirname(os.path.realpath(__file__)), "ERROR.log"))
    translator.set_logger_logging(True, True, True, False)

    # do the translation
    if os.path.exists(in_cif):
        if os.path.exists(out_dir):
            translator.translate_and_validate(in_cif, out_xml, schema)
            if os.path.exists(out_dir):
                print("Output XML file", out_xml, "has been created")
            translator.write_logger_logs(True, True, True)
        else:
            print("Output file path", out_xml, "does not exit")
    else:
        print("Input file", in_cif, "does not exist")


if __name__ == "__main__":
    main()



#!/usr/bin/env python
import mrcfile
from Bio.PDB.MMCIF2Dict import MMCIF2Dict
import os
import sys
import argparse
import math
import numpy as np
import json


def get_mapheader(map_fullname):
    header = mrcfile.open(map_fullname, permissive=True, header_only=True)
    return header.header


def get_model(model_fullname):
    """

    :param model_fullname: String of the model name
    :return: Biopython model object
    """

    mmcif_dict = MMCIF2Dict(model_fullname)
    structure = []
    x = mmcif_dict['_atom_site.Cartn_x']
    y = mmcif_dict['_atom_site.Cartn_y']
    z = mmcif_dict['_atom_site.Cartn_z']
    for x, y, z in zip(x, y, z):
        try:
            newx = float(x)
            newy = float(y)
            newz = float(z)
            structure.append((newx, newy, newz))
        except ValueError:
            continue

    return structure


def header_check(header):
    crs = (header.mapc, header.mapr, header.maps)
    crsindices = (crs.index(1), crs.index(2), crs.index(3))
    nstarts = (header.nxstart, header.nystart, header.nzstart)
    nxyz = (header.nx, header.ny, header.nz)

    if crs != (1, 2, 3):
        nxyz = (nxyz[crsindices[0]], nxyz[crsindices[1]], nxyz[crsindices[2]])
        nstarts = (nstarts[crsindices[0]], nstarts[crsindices[1]], nstarts[crsindices[2]])

    return nxyz, nstarts


def matrix_indices(nstarts, apixs, onecoor, header):
    """

        Method 2: using the fractional coordinate matrix to calculate the indices when the maps are non-orthogonal

    :return:
    """

    angs = [header.cellb.alpha, header.cellb.beta, header.cellb.gamma]
    matrix = map_matrix(apixs, angs)
    result = matrix.dot(np.asarray(onecoor))
    xindex = result[0] - nstarts[0]
    yindex = result[1] - nstarts[1]
    zindex = result[2] - nstarts[2]

    return (xindex, yindex, zindex)


def map_matrix(apixs, angs):
    """

        calculate the matrix to transform Cartesian coordinates to fractional coordinates
        (check the definination to see the matrix formular)

    :param apixs: array of apix lenght
    :param angs: array of anglex in alpha, beta, gamma order
    :return:
    """

    ang = (angs[0] * math.pi / 180, angs[1] * math.pi / 180, angs[2] * math.pi / 180)
    insidesqrt = 1 + 2 * math.cos(ang[0]) * math.cos(ang[1]) * math.cos(ang[2]) - \
        math.cos(ang[0]) ** 2 - \
        math.cos(ang[1]) ** 2 - \
        math.cos(ang[2]) ** 2

    cellvolume = apixs[0] * apixs[1] * apixs[2] * math.sqrt(insidesqrt)

    m11 = 1 / apixs[0]
    m12 = - math.cos(ang[2]) / (apixs[0] * math.sin(ang[2]))

    m13 = apixs[1] * apixs[2] * (math.cos(ang[0]) * math.cos(ang[2]) - math.cos(ang[1])) / (
        cellvolume * math.sin(ang[2]))
    m21 = 0
    m22 = 1 / (apixs[1] * math.sin(ang[2]))
    m23 = apixs[0] * apixs[2] * (math.cos(ang[1]) * math.cos(ang[2]) - math.cos(ang[0])) / (
        cellvolume * math.sin(ang[2]))
    m31 = 0
    m32 = 0
    m33 = apixs[0] * apixs[1] * math.sin(ang[2]) / cellvolume
    prematrix = [[m11, m12, m13], [m21, m22, m23], [m31, m32, m33]]
    matrix = np.asarray(prematrix)

    return matrix


def get_indices(header, onecoor):
    """
    Find one atom's indices correspoding to its cubic or plane
    the 8 (cubic) or 4 (plane) indices are saved in indices variable
    :param map: Density map instance from TEMPy.MapParser
    :param onecoor: List contains the atom coordinates in (x, y, z) order
    :return: Tuple contains two list of index: first has the 8 or 4 indices in the cubic;
             second has the float index of the input atom
    """

    zdim = header.cella.z
    znintervals = header.mz
    z_apix = zdim / znintervals

    ydim = header.cella.y
    ynintervals = header.my
    y_apix = ydim / ynintervals

    xdim = header.cella.x
    xnintervals = header.mx
    x_apix = xdim / xnintervals

    nxyzstart = header_check(header)
    nxstart, nystart, nzstart = nxyzstart[1]

    if header.cellb.alpha == header.cellb.beta == header.cellb.gamma == 90.:
        zindex = float(onecoor[2] - header.origin.z) / z_apix - nzstart
        yindex = float(onecoor[1] - header.origin.y) / y_apix - nystart
        xindex = float(onecoor[0] - header.origin.x) / x_apix - nxstart
    else:
        apixs = [x_apix, y_apix, z_apix]
        xindex, yindex, zindex = matrix_indices(nxyzstart[1], apixs, onecoor, header)

    oneindex = [xindex, yindex, zindex]

    return oneindex, nxyzstart[0]


def check(model, header):
    atoms_outside_num = 0
    atom_counter = 0
    for atom in model:
        atom_counter += 1
        oneindex, nxyz = get_indices(header, atom)
        if oneindex[0] > nxyz[0] - 1 or oneindex[0] < 0 or \
                oneindex[1] > nxyz[1] - 1 or oneindex[1] < 0 or \
                oneindex[2] > nxyz[2] - 1 or oneindex[2] < 0:
            atoms_outside_num += 1

    atom_outside_fraction = atoms_outside_num / atom_counter if atom_counter != 0 else 0
    return atoms_outside_num, atom_outside_fraction


def read_args():
    """
        Read arguments
    :return: map, model
    """

    assert len(sys.argv) > 1, ('There has to be arguments for the commands.\n \
                              Usage main.py -m/--map  <fullmapname> -d/--model <fullmodelname>')
    parser = argparse.ArgumentParser(description='Input map and model')
    parser.add_argument('--map', '-m', nargs='?', help='EM volume')
    parser.add_argument('--model', '-d', nargs='?', help='Model')
    parser.add_argument('--output', '-o', nargs='?', help='Output JSON file')
    args = parser.parse_args()
    return args.map, args.model, args.output


def main():
    input_map, input_model, output_file = read_args()
    if os.path.isfile(input_map) and os.path.isfile(input_model):
        header = get_mapheader(input_map)
        model = get_model(input_model)
        result = check(model, header)
        result_dict = {
            'em_volume': os.path.basename(input_map),
            'model': os.path.basename(input_model),
            'atomsoutside': result[0],
            'fraction': result[1]
        }

        if output_file:
            with open(output_file, "w") as fw:
                json.dump(result_dict, fw)

        result_str = 'map:                           {}\n' \
                     'model:                         {}\n' \
                     'Number of atoms outside box:   {}\n' \
                     'Fraction of atoms outside box: {}'.format(os.path.basename(input_map),
                                                                os.path.basename(input_model), result[0], result[1])
        print(result_str)
        print('---------------------------------------')
    else:
        print('Please provide both volume map and model.')

        if output_file:
            result_dict = {
                'em_volume': '',
                'model': '',
                'atomsoutside': 0,
                'fraction': 0
            }
            with open(output_file, "w") as fw:
                json.dump(result_dict, fw)


if __name__ == '__main__':
    main()
